@@ -1,4 +1,4 @@
-/* 01jan14abu
+/* 16mar14abu
  * (c) Software Lab. Alexander Burger
  */
 
@@ -17,7 +17,11 @@ function formKey(event) {
    if (Hint  &&  Hint.style.visibility == "visible") {
       if ((Item >= 0 && Key == 13) || Key == 38 || Key == 40)
          return false;
-      if (event.keyCode == 27) {
+      if (Key == 13) {
+         Hint.style.visibility = "hidden";
+         return true;
+      }
+      if (Key == 27) {
          Hint.style.visibility = "hidden";
          return false;
       }
@@ -215,6 +219,8 @@ function post(form, auto, exe, file) {
                      fld.src = val;
                   else if (fld.value != val) {
                      fld.value = val;
+                     if (fld.pilSetValue)
+                        fld.pilSetValue(val);
                      fld.scrollTop = fld.scrollHeight;
                   }
                   fld.disabled = false;
@@ -230,6 +236,8 @@ function post(form, auto, exe, file) {
                         document.getElementsByName(fld.name)[0].value = "T";
                      ++i;
                   }
+                  if (fld.pilDisable)
+                     fld.pilDisable(fld.disabled);
                }
                while (i < txt.length && (j = "#*?".indexOf(txt[i].charAt(0))) >= 0) {
                   switch (j) {
@@ -241,7 +249,7 @@ function post(form, auto, exe, file) {
                      if ((cls = fld.getAttribute("class")) != null) {
                         j = cls.indexOf("  ");
                         if (!val)
-                           val = j >= 0? cls.substr(j+2) : cls;
+                           val = j >= 0? cls.substr(j+2) : "";
                         else if (j >= 0)
                            val += cls.substr(j);
                         else
@@ -472,7 +480,7 @@ function addHint(i, field, str) {
          Hint.style.visibility = "hidden";
       }
       field.onchange = function() {
-         return fldChg(field, item);
+         return fldChg(field);
       };
       Item = -1;
    }
@@ -569,3 +577,10 @@ function lispVal(x) {
       return "%2D" + encodeURIComponent(x.substr(1));
    return encodeURIComponent(x);
 }
+
+function ping(min) {
+   if (SesId) {
+      lisp(null, "ping", min);
+      setTimeout(function() {ping(min)}, 20000);
+   }
+}
