@@ -1,4 +1,4 @@
-/* 23jun13abu
+/* 10mar14abu
  * (c) Software Lab. Alexander Burger
  */
 
@@ -171,6 +171,7 @@ void setRaw(void) {
    if (Tio && !Termio) {
       *(Termio = malloc(sizeof(struct termios))) = OrgTermio;
       Termio->c_iflag = 0;
+      Termio->c_oflag = OPOST+ONLCR;
       Termio->c_lflag = ISIG;
       Termio->c_cc[VMIN] = 1;
       Termio->c_cc[VTIME] = 0;
@@ -215,6 +216,17 @@ any doSigio(any ex) {
    return x;
 }
 
+// (kids) -> lst
+any doKids(any ex __attribute__((unused))) {
+   int i;
+   any x;
+
+   for (i = 0, x = Nil; i < Children; ++i)
+      if (Child[i].pid)
+         x = cons(box(Child[i].pid * 2), x);
+   return x;
+}
+
 // (protect . prg) -> any
 any doProtect(any x) {
    ++Env.protect;
@@ -277,7 +289,7 @@ any doEnv(any x) {
 
    Push(c1, Nil);
    if (!isCell(x = cdr(x))) {
-      for (p = Break? Env.bind->link : Env.bind;  p;  p = p->link) {
+      for (p = Env.bind;  p;  p = p->link) {
          if (p->i == 0) {
             for (i = p->cnt;  --i >= 0;) {
                for (x = data(c1); ; x = cdr(x)) {
@@ -326,7 +338,7 @@ any doUp(any x) {
       cnt = 1;
    else
       cnt = (int)unBox(y),  x = cdr(x),  y = car(x);
-   for (p = Break? Env.bind->link : Env.bind, val = &val(y);  p;  p = p->link) {
+   for (p = Env.bind, val = &val(y);  p;  p = p->link) {
       if (p->i <= 0) {
          for (i = 0;  i < p->cnt;  ++i)
             if (p->bnd[i].sym == y) {
