@@ -1,4 +1,4 @@
-/* 28aug13abu
+/* 25mar14abu
  * (c) Software Lab. Alexander Burger
  */
 
@@ -141,6 +141,7 @@ static symInit Symbols[] = {
    {doFree, "free"},
    {doFrom, "from"},
    {doFull, "full"},
+   {doFully, "fully"},
    {doFunQ, "fun?"},
    {doGc, "gc"},
    {doGe, ">="},
@@ -172,6 +173,7 @@ static symInit Symbols[] = {
    {doJob, "job"},
    {doJournal, "journal"},
    {doKey, "key"},
+   {doKids, "kids"},
    {doKill, "kill"},
    {doLast, "last"},
    {doLe, "<="},
